@@ -3,30 +3,30 @@ use std::convert::TryInto;
 
 use crate::compressor::Compressor;
 use crate::decompressor::Decompressor;
-use crate::types::{DataType, NumberLike};
+use crate::types::NumberLike;
 
 impl NumberLike for bool {
-  fn num_eq(&self, other: &Self) -> bool {    
+  const HEADER_BYTE: u8 = 7;
+  // it's easiest to use 8 bits per uncompressed boolean
+  // because that's how rust represents them too
+  const PHYSICAL_BITS: usize = 8;
+
+  type Diff = u8;
+
+  fn num_eq(&self, other: &Self) -> bool {
     self.eq(other)
   }
 
   fn num_cmp(&self, other: &Self) -> Ordering {
     self.cmp(other)
   }
-}
 
-pub struct BoolDataType {}
-
-impl DataType<bool> for BoolDataType {
-  const HEADER_BYTE: u8 = 7;
-  const BIT_SIZE: usize = 8;
-
-  fn offset_diff(upper: bool, lower: bool) -> u64 {
-    (upper as u64) - (lower as u64)
+  fn offset_diff(upper: bool, lower: bool) -> u8 {
+    (upper as u8) - (lower as u8)
   }
 
-  fn add_offset(lower: bool, off: u64) -> bool {
-    (lower as u64 + off) != 0
+  fn add_offset(lower: bool, off: u8) -> bool {
+    lower || (off > 0)
   }
 
   fn bytes_from(value: bool) -> Vec<u8> {
@@ -38,5 +38,5 @@ impl DataType<bool> for BoolDataType {
   }
 }
 
-pub type BoolCompressor = Compressor<u8, BoolDataType>;
-pub type BoolDecompressor = Decompressor<u8, BoolDataType>;
+pub type BoolCompressor = Compressor<u8>;
+pub type BoolDecompressor = Decompressor<u8>;
