@@ -3,9 +3,14 @@ use std::convert::TryInto;
 
 use crate::compressor::Compressor;
 use crate::decompressor::Decompressor;
-use crate::types::{DataType, NumberLike};
+use crate::types::NumberLike;
 
 impl NumberLike for u32 {
+  const HEADER_BYTE: u8 = 4;
+  const PHYSICAL_BITS: usize = 32;
+
+  type Diff = u32;
+
   fn num_eq(&self, other: &Self) -> bool {
     self.eq(other)
   }
@@ -13,20 +18,13 @@ impl NumberLike for u32 {
   fn num_cmp(&self, other: &Self) -> Ordering {
     self.cmp(other)
   }
-}
 
-pub struct U32DataType {}
-
-impl DataType<u32> for U32DataType {
-  const HEADER_BYTE: u8 = 4;
-  const BIT_SIZE: usize = 32;
-
-  fn offset_diff(upper: u32, lower: u32) -> u64 {
-    (upper - lower) as u64
+  fn offset_diff(upper: u32, lower: u32) -> u32 {
+    upper - lower
   }
 
-  fn add_offset(lower: u32, off: u64) -> u32 {
-    lower + off as u32
+  fn add_offset(lower: u32, off: u32) -> u32 {
+    lower + off
   }
 
   fn bytes_from(num: u32) -> Vec<u8> {
@@ -38,5 +36,5 @@ impl DataType<u32> for U32DataType {
   }
 }
 
-pub type U32Compressor = Compressor<u32, U32DataType>;
-pub type U32Decompressor = Decompressor<u32, U32DataType>;
+pub type U32Compressor = Compressor<u32>;
+pub type U32Decompressor = Decompressor<u32>;
