@@ -3,9 +3,14 @@ use std::convert::TryInto;
 
 use crate::compressor::Compressor;
 use crate::decompressor::Decompressor;
-use crate::types::{DataType, NumberLike};
+use crate::types::NumberLike;
 
 impl NumberLike for u64 {
+  const HEADER_BYTE: u8 = 2;
+  const PHYSICAL_BITS: usize = 64;
+
+  type Diff = u64;
+
   fn num_eq(&self, other: &Self) -> bool {
     self.eq(other)
   }
@@ -13,13 +18,6 @@ impl NumberLike for u64 {
   fn num_cmp(&self, other: &Self) -> Ordering {
     self.cmp(other)
   }
-}
-
-pub struct U64DataType {}
-
-impl DataType<u64> for U64DataType {
-  const HEADER_BYTE: u8 = 2;
-  const BIT_SIZE: usize = 64;
 
   fn offset_diff(upper: u64, lower: u64) -> u64 {
     upper - lower
@@ -38,5 +36,5 @@ impl DataType<u64> for U64DataType {
   }
 }
 
-pub type U64Compressor = Compressor<u64, U64DataType>;
-pub type U64Decompressor = Decompressor<u64, U64DataType>;
+pub type U64Compressor = Compressor<u64>;
+pub type U64Decompressor = Decompressor<u64>;
